# -*- coding: utf-8 -*-

from Handlers import BaseHandler
# -------------------------------------------------------------------------------
#
# -------------------------------------------------------------------------------


class NAT(BaseHandler.BaseHandler):
    # =======================================================
    #
    # =======================================================
    def __init__(self):
        self.name = 'NAT'
        self.interfaces = ['eth0', 'eth1', 'br0', 'ppp0']
        self.zones = ['external', 'trusted', 'internal']
        self.configurable_zones = ['external', 'internal']
    # =======================================================
    #
    # =======================================================
    def Get(self, request):

        status = 200
        return self.zoneInfo(), status

    # =======================================================
    #
    # =======================================================
    def Put(self, request):

        status = 200
        self.setRules(request.json['rules'])
        return status

    # =======================================================
    #
    # =======================================================
    def zoneInfo(self):

        zone_info = {'interfaces': {}, 'rules': {},
                     'zones': self.configurable_zones}
        for zone in self.configurable_zones:
            zone_info['rules'][zone] = []

        result = self.firewallCmd("list-all-zones")
        if result:
            lines = result.split('\n')
            zone = False
            for line in lines:
                if not line.startswith('\t') and not line.startswith(' '):
                    zone = line.split(' ')[0]
                    if zone in self.configurable_zones:
                        rules_flg = False
                        configurable_flg = True
                    elif zone in self.zones:
                        configurable_flg = False
                    else:
                        zone = False
                elif zone:
                    line = line.strip(' ')
                    if line.startswith('interfaces:'):
                        interfaces = line.split(':')[1].split(' ')
                        for interface in interfaces:
                            if interface in self.interfaces:
                                zone_info['interfaces'][interface] = zone

                    if configurable_flg:
                        if line.startswith('rich rules:'):
                            rules_flg = True

                        elif rules_flg:
                            rule_dict = {'srcAddress': '', 'destAddress': '',
                                         'destPort': '', 'toAddress': '', 'toPort': '', 'protocol': ''}
                            if 'forward' in line:
                                back_param = ""
                                rule = line.strip('\t').split(' ')
                                for param in rule:
                                    if param.startswith('address='):
                                        addr = param.strip(
                                            'address=').strip('"')
                                        if back_param == "source":
                                            rule_dict['srcAddress'] = addr
                                        elif back_param == "destination":
                                            rule_dict['destAddress'] = addr
                                    elif param.startswith('port='):
                                        rule_dict['destPort'] = param.strip(
                                            'port=').strip('"')
                                    elif param.startswith('to-addr='):
                                        rule_dict['toAddress'] = param.strip(
                                            'to-addr=').strip('"')
                                    elif param.startswith('to-port='):
                                        rule_dict['toPort'] = param.strip(
                                            'to-port=').strip('"')
                                    elif param.startswith('protocol='):
                                        rule_dict['protocol'] = param.strip(
                                            'protocol=').strip('"')
                                    back_param = param

                                zone_info['rules'][zone].append(rule_dict)
        print(zone_info)
        return zone_info

    # =======================================================
    #
    # =======================================================
    def setRules(self, args):

        print(args)

        for zone in args:
            now_rules = self.firewallCmd(
                "list-rich-rules --zone=%s" % zone).strip('\n').split('\n')
            for rule in args[zone]:
                add_rule = []
                add_rule.append('rule family=\"ipv4\"')
                if 'srcAddress' in rule and rule['srcAddress']:
                    add_rule.append('source address=\"%s\"' %
                                    rule['srcAddress'])
                if 'destAddress' in rule and rule['destAddress']:
                    add_rule.append('destination address=\"%s\"' %
                                    rule['destAddress'])
                if 'destPort' in rule and rule['destPort']:
                    add_rule.append('forward-port port=\"%s\"' %
                                    rule['destPort'])
                if 'protocol' in rule and rule['protocol']:
                    add_rule.append('protocol=\"%s\"' % rule['protocol'])
                if 'toPort' in rule and rule['toPort']:
                    add_rule.append('to-port=\"%s\"' % rule['toPort'])
                if 'toAddress' in rule and rule['toAddress']:
                    add_rule.append('to-addr=\"%s\"' % rule['toAddress'])

                rule_str = ' '.join(add_rule)
                if rule_str in now_rules:
                    now_rules.remove(rule_str)
                else:
                    self.firewallCmd("add-rich-rule='%s' --zone=%s" %
                                     (rule_str, zone), True)
            if now_rules:
                for rule in now_rules:
                    if 'forward' in rule:
                        self.firewallCmd(
                            "remove-rich-rule='%s' --zone=%s" % (rule, zone), True)
